@@ -17,25 +17,31 @@ from bitcoin.serialize import (
     serialize_hash, deserialize_hash,
     serialize_list, deserialize_list)
 
-from .tools import StringIO, icmp, list, tuple
+from .tools import Bits, StringIO, icmp, list, tuple
 
 SENTINAL = object()
 
 # ===----------------------------------------------------------------------===
 
+import numbers
+
 class PatriciaLink(SerializableMixin, HashableMixin):
-    __slots__ = ('prefix', 'node', '_hash')
+    __slots__ = 'prefix node _hash'.split()
 
-    def __init__(self, prefix, node, _hash=None, *args, **kwargs):
+    def __init__(self, prefix, node=None, hash=None, *args, **kwargs):
+        if not isinstance(prefix, Bits):
+            if isinstance(prefix, six.binary_type):
+                prefix = Bits(bytes=prefix)
+            else:
+                prefix = Bits(prefix)
+        if isinstance(node, numbers.Integral):
+            node, hash = hash, node
         super(PatriciaLink, self).__init__(*args, **kwargs)
-        self.prefix, self.node, self._hash = prefix, node, _hash
+        self.prefix, self.node, self._hash = prefix, node, hash
 
-    def __lt__(self, other): return (self.prefix, self.node) <  (other.prefix, other.node)
-    def __le__(self, other): return (self.prefix, self.node) <= (other.prefix, other.node)
-    def __eq__(self, other): return (self.prefix, self.node) == (other.prefix, other.node)
-    def __ne__(self, other): return (self.prefix, self.node) != (other.prefix, other.node)
-    def __ge__(self, other): return (self.prefix, self.node) >= (other.prefix, other.node)
-    def __gt__(self, other): return (self.prefix, self.node) >  (other.prefix, other.node)
+    @property
+    def pruned(self):
+        return self.node is None
 
     def serialize(self, digest=False):
         result = serialize_varchar(self.prefix)
@@ -66,24 +72,51 @@ class PatriciaLink(SerializableMixin, HashableMixin):
         self.hash__setter(value)
         return value
 
+    def _thunk_other(inner):
+        def outer(self, other):
+            def _prefix(node):
+                return getattr(node, 'prefix', NotImplemented)
+            def _node(node):
+                return getattr(node, 'node', NotImplemented)
+            def _hash(node):
+                return getattr(node, 'hash', NotImplemented)
+            def _icmp():
+                return icmp(
+                    (x(self)  for x in (_prefix, _node, _hash)),
+                    (x(other) for x in (_prefix, _node, _hash)))
+            return inner(self, icmp=_icmp)
+        return outer
+    __lt__ = _thunk_other(lambda self,icmp:icmp() <  0)
+    __le__ = _thunk_other(lambda self,icmp:icmp() <= 0)
+    __eq__ = _thunk_other(lambda self,icmp:icmp() == 0)
+    __ne__ = _thunk_other(lambda self,icmp:icmp() != 0)
+    __ge__ = _thunk_other(lambda self,icmp:icmp() >= 0)
+    __gt__ = _thunk_other(lambda self,icmp:icmp() >  0)
+
     def __repr__(self):
-        return ('%s(prefix=\'%s\'.decode(\'hex\'), '
-                   'node=0x%064x)') % (
-            self.__class__.__name__,
-            self.prefix.encode('hex'),
-            self.hash or 0)
+        parts = []
+        parts.append('prefix=%s' % repr(self.prefix))
+        parts.append('hash=0x%064x' % (self.hash or 0))
+        return '%s(%s)' % (self.__class__.__name__, ', '.join(parts))
 
 # ===----------------------------------------------------------------------===
 
+import operator
 from bisect import bisect_left
 from os.path import commonprefix
+from struct import pack, unpack
 class PatriciaNode(SerializableMixin, HashableMixin):
     """An ordered dictionary implemented with a hybrid level- and node-
     compressed prefix tree."""
-    __slots__ = 'value extra children _hash size length'.split()
+    __slots__ = 'value children _hash size length'.split()
 
-    HAS_VALUE = 0x1
-    HAS_EXTRA = 0x2
+    OFFSET_LEFT  = 0
+    OFFSET_RIGHT = 2
+    HAS_VALUE    = 4
+    HASH_MASK    = 0x1f
+    PRUNE_LEFT   = 5
+    PRUNE_RIGHT  = 6
+    PRUNE_VALUE  = 7
 
     @classmethod
     def _get_attr_class(cls, attr):
@@ -101,8 +134,13 @@ class PatriciaNode(SerializableMixin, HashableMixin):
             if six.callable(serialize):
                 return serialize(elem)
         return elem
-    _prepare_key = lambda cls,elem:cls._prepare(elem, 'key')
-    _prepare_value = lambda cls,elem:cls._prepare(elem, 'value')
+    @classmethod
+    def _prepare_key(cls, elem):
+        elem = cls._prepare(elem, 'key')
+        if isinstance(elem, six.binary_type):
+            elem = Bits(bytes=elem)
+        return elem
+    _prepare_value = classmethod(lambda cls,elem:cls._prepare(elem, 'value'))
 
     @classmethod
     def _unpickle(cls, string, attr):
@@ -112,70 +150,137 @@ class PatriciaNode(SerializableMixin, HashableMixin):
             return deserialize(StringIO(string))
         else:
             return attr_class(string)
-    _unpickle_key = lambda cls,string:cls._unpickle(string, 'key')
-    _unpickle_value = lambda cls,string:cls._unpickle(string, 'value')
+    @classmethod
+    def _unpickle_key(cls, string):
+        string = getattr(string, 'bytes', string)
+        return cls._unpickle(string, 'key')
+    _unpickle_value = classmethod(lambda cls,string:cls._unpickle(string, 'value'))
+
+    def __init__(self, value=None, children=None, prune_value=None, *args, **kwargs):
+        if value is not None:
+            if not isinstance(value, six.binary_type):
+                raise TypeError(u"value must be a binary string, or None")
+            if prune_value is None:
+                prune_value = False
+        else:
+            assert prune_value is None
 
-    def __init__(self, value=None, extra=None, children=None, *args, **kwargs):
-        if value is not None and not isinstance(value, six.binary_type):
-            raise TypeError(u"value must be a binary string, or None")
-        if extra is not None and not isinstance(extra, six.binary_type):
-            raise TypeError(u"extra must be a binary string, or None")
         if children is None:
             children = tuple()
         link_class = getattr(self, 'get_link_class',
             lambda: getattr(self, 'link_class', PatriciaLink))()
         if hasattr(children, 'keys'):
-            children = tuple(
-                link_class(prefix=prefix, node=children[prefix])
-                for prefix in sorted(children))
+            _children = list()
+            for prefix in children:
+                if not isinstance(children[prefix], numbers.Integral):
+                    _children.append(link_class(prefix=prefix, node=children[prefix]))
+                else:
+                    _children.append(link_class(prefix=prefix, hash=children[prefix]))
+            children = _children
+        else:
+            _children = list()
+            for child in children:
+                if not all(hasattr(child, x) for x in ('prefix', 'node', 'hash')):
+                    child = link_class(*child)
+                _children.append(child)
+            children = _children
+        children = sorted(children)
+
+        assert 0 <= len(children) <= 2
+        for link in children:
+            assert 0 < len(link.prefix) < 2**64
+        assert len(children) == len(set(l.prefix[0] for l in children))
 
         super(PatriciaNode, self).__init__(*args, **kwargs)
 
-        self.value = value
-        self.extra = extra
+        self.value, self.prune_value = value, prune_value
         getattr(self, 'children_create', lambda:setattr(self, 'children', list()))()
-        self.children.extend(children)
-        self.size = int(value is not None)
-        self.length = self.size
-        for child in children:
-            self.size += child.node.size
-            self.length += child.node.length
+        self.children.extend(sorted(children))
+
+        size = int(value is not None)
+        length = size - int(prune_value is True)
+        for link in children:
+            if not link.pruned:
+                size += link.node.size
+                length += link.node.length
+        self.size, self.length = size, length
+
+    @property
+    def left(self):
+        if self.children and self.children[0].prefix[0] is False:
+            return self.children[0]
+        else:
+            return None
+
+    @property
+    def right(self):
+        if len(self.children)==2:
+            return self.children[1]
+        elif len(self.children)==1 and self.children[0].prefix[0] is True:
+            return self.children[0]
+        else:
+            return None
 
     @property
     def flags(self):
-        flags = 0
-        if self.value is not None: flags |= self.HAS_VALUE
-        if self.extra is not None: flags |= self.HAS_EXTRA
+        flags = reduce(operator.or_, (
+            int(self.value is not None)                       << self.HAS_VALUE,
+            int(getattr(self.left,  'pruned', False) is True) << self.PRUNE_LEFT,
+            int(getattr(self.right, 'pruned', False) is True) << self.PRUNE_RIGHT,
+            int(        self.prune_value             is True) << self.PRUNE_VALUE,))
+        if self.left is not None:
+            flags |= min(3, len(self.left.prefix))  << self.OFFSET_LEFT
+        if self.right is not None:
+            flags |= min(3, len(self.right.prefix)) << self.OFFSET_RIGHT
         return flags
 
     def serialize(self, digest=False):
+        def _serialize_link(link):
+            len_ = len(link.prefix)
+            if len_ >= 3:
+                parts.append(serialize_varint(len_-3))
+            parts.append(link.prefix[1:].tobytes())
+            if digest or link.pruned:
+                parts.append(serialize_hash(link.hash, 32))
+            else:
+                parts.append(link.node.serialize(digest=digest))
+        parts = []
         flags = self.flags
-        result = serialize_varint(flags)
-        result += serialize_list(self.children,
-            lambda l:l.serialize(digest=digest))
-        if flags & self.HAS_VALUE:
-            result += serialize_varchar(self.value)
-        if flags & self.HAS_EXTRA:
-            result += serialize_varchar(self.extra)
-        return result
+        if digest:
+            flags &= self.HASH_MASK
+        parts.append(pack('B', flags))
+        if self.left is not None:
+            _serialize_link(self.left)
+        if self.right is not None:
+            _serialize_link(self.right)
+        if self.value is not None:
+            parts.append(serialize_varchar(self.value))
+        return b''.join(parts)
     @classmethod
-    def deserialize_link(cls, file_, nodes, *args, **kwargs):
-        prefix = deserialize_varchar(file_)
-        hash = deserialize_hash(file_, 32)
-        return getattr(cls, 'get_link_class',
-            lambda: getattr(cls, 'link_class', PatriciaLink)
-            )(prefix=prefix, node=nodes[hash], _hash=hash, *args, **kwargs)
-    @classmethod
-    def deserialize(cls, file_, nodes=None, *args, **kwargs):
-        nodes = nodes or {}
+    def deserialize(cls, file_):
         initargs = {}
         flags = deserialize_varint(file_)
-        initargs['children'] = deserialize_list(file_,
-            lambda x:cls.deserialize_link(x, nodes))
-        if flags & cls.HAS_VALUE:
+        initargs['children'] = {}
+        def _deserialize_branch(attr, implicit, bitlength, prune):
+            if not bitlength:
+                return
+            if bitlength == 3:
+                bitlength = deserialize_varint(file_) + 3
+            bytelength = (bitlength + 7) // 8
+            bytes_ = file_.read(bytelength)
+            assert len(bytes_) == bytelength
+            prefix = implicit + Bits(bytes=bytes_, length=bitlength-1)
+            if prune:
+                initargs['children'][prefix] = deserialize_hash(file_, 32)
+            else:
+                initargs['children'][prefix] = cls.deserialize(file_)
+        _deserialize_branch('left', Bits([False]),
+            (flags >> cls.OFFSET_LEFT)  & 3, bool(flags & (1 << cls.PRUNE_LEFT)),)
+        _deserialize_branch('right', Bits([True]),
+            (flags >> cls.OFFSET_RIGHT) & 3, bool(flags & (1 << cls.PRUNE_RIGHT)),)
+        if flags & (1 << cls.HAS_VALUE):
             initargs['value'] = deserialize_varchar(file_)
-        if flags & cls.HAS_EXTRA:
-            initargs['extra'] = deserialize_varchar(file_)
+            initargs['prune_value'] = bool(flags & (1 << cls.PRUNE_VALUE))
         return cls(**initargs)
 
     def hash__getter(self):
@@ -186,42 +291,63 @@ class PatriciaNode(SerializableMixin, HashableMixin):
         return self.hash
 
     def __repr__(self):
+        parts = []
         if self.value is not None:
-            value_str = 'value=\'%s\'.decode(\'hex\'), ' % self.value.encode('hex')
-        else:
-            value_str = ''
-        if self.extra is not None:
-            extra_str = 'extra=\'%s\'.decode(\'hex\'), ' % self.extra.encode('hex')
-        else:
-            extra_str = ''
-        return ('%s(%s%schildren=[%s])' % (
-            self.__class__.__name__,
-            value_str,
-            extra_str,
-            ', '.join(map(repr, self.children))))
+            parts.append('\'%s\'.decode(\'hex\')' % self.value.encode('hex'))
+            if self.prune_value:
+                parts.append('prune_value=True')
+        parts.append('children=[%s]' % ', '.join(map(repr, self.children)))
+        return '%s(%s)' % (self.__class__.__name__, ', '.join(parts))
 
     def __nonzero__(self):
         "x.__nonzero__() <==> bool(x)"
         return bool(self.length)
+    __bool__ = __nonzero__
 
-    def __lt__(self, other):
+    def _thunk_other(inner):
+        def outer(self, other):
+            def _hash():
+                return getattr(other, 'hash', NotImplemented)
+            def _icmp():
+                if isinstance(other, PatriciaNode):
+                    return icmp(self.iteritems(), other.iteritems())
+                elif hasattr(other, 'keys'):
+                    return icmp(self.iteritems(), sorted((x,other[x]) for x in other))
+                else:
+                    try:
+                        return icmp(self.iteritems(), sorted(other))
+                    except TypeError:
+                        return NotImplemented
+            return inner(self, _hash, _icmp)
+        return outer
+    @_thunk_other
+    def __lt__(self, hash, icmp):
         "x.__lt__(o) <==> x < o"
-        return icmp(self.iteritems(), other.iteritems()) < 0
-    def __le__(self, other):
-        "x.__lt__(o) <==> x < o"
-        return self.hash == other.hash or self < other
-    def __eq__(self, other):
+        return icmp() < 0
+    @_thunk_other
+    def __le__(self, hash, icmp):
+        "x.__le__(o) <==> x <0 o"
+        return self.hash == hash() or icmp() <= 0
+    @_thunk_other
+    def __eq__(self, hash, icmp):
         "x.__eq__(o) <==> x == o"
-        return self.hash == other.hash
+        result = hash()
+        if result is NotImplemented:
+            result = icmp() == 0
+        else:
+            result = result == self.hash
+        return result
     def __ne__(self, other):
-        "x.__eq__(o) <==> x != o"
-        return self.hash != other.hash
-    def __ge__(self, other):
-        "x.__lt__(o) <==> x < o"
-        return self.hash == other.hash or self > other
-    def __gt__(self, other):
-        "x.__lt__(o) <==> x < o"
-        return icmp(self.iteritems(), other.iteritems()) > 0
+        "x.__ne__(o) <==> x != o"
+        return not (self == other)
+    @_thunk_other
+    def __ge__(self, hash, icmp):
+        "x.__ge__(o) <==> x >= o"
+        return self.hash == hash() or icmp() >= 0
+    @_thunk_other
+    def __gt__(self, hash, icmp):
+        "x.__gt__(o) <==> x > o"
+        return icmp() > 0
 
     def __len__(self):
         "x.__len__() <==> len(x)"
@@ -372,10 +498,10 @@ class PatriciaNode(SerializableMixin, HashableMixin):
             lambda: getattr(self, 'node_class', self.__class__))()
 
         # TODO: Maybe remove this and rely on duck typing?
-        if not (isinstance(key, six.binary_type) and
+        if not (isinstance(key, Bits) and
                 isinstance(value, six.binary_type)):
-            raise TypeError(u"%s can only map binary string -> binary "
-                u"string" % self.__class__.__name__)
+            raise TypeError(u"%s can only map bitstring -> binary type"
+                % self.__class__.__name__)
 
         path = list()
         prefix, old_node = self._get_node_by_key(key, path=path)
@@ -406,14 +532,14 @@ class PatriciaNode(SerializableMixin, HashableMixin):
                     children = (link_class(
                         prefix = old_node.children[idx].prefix[len(common_prefix):],
                         node   = old_node.children[idx].node,
-                        _hash  = old_node.children[idx]._hash),))
+                        hash   = old_node.children[idx]._hash),))
                 new_node = node_class(
                     value    = old_node.value,
                     children = (list(x for x in old_node.children[:idx]) +
                                 list((link_class(prefix=common_prefix, node=inner_node),)) +
                                 list(x for x in old_node.children[idx+1:])))
 
-            elif common_prefix:
+            elif len(common_prefix):
                 leaf_node = node_class(
                     value    = value,
                     children = {})
@@ -502,7 +628,7 @@ class PatriciaNode(SerializableMixin, HashableMixin):
                             list((link_class(
                                 prefix = prefix + new_node.children[0].prefix,
                                 node   = new_node.children[0].node,
-                                _hash  = new_node.children[0]._hash),)) +
+                                hash   = new_node.children[0]._hash),)) +
                             list(x for x in parent.children[idx+1:])))
 
         self._propogate(new_node, path=path)
